import * as childProcess from 'child_process';
import * as fs from 'fs';
import * as path from 'path';

const processPath = path.join(__dirname, '../../lib/patcher-installer');

export default function launchProcess(
  diskFile: string,
  offset: number,
  compressedSize: number,
  decryptionKeys: [number, number, number] | undefined,
  previousFile: string | undefined,
  outputStream: fs.WriteStream,
) {
  return new Promise((resolve, reject) => {
    const parameters = [
      '--disk', diskFile,
      '--offset', offset,
      '--size', compressedSize,
    ];
    if (decryptionKeys !== undefined) {
      parameters.push('--keys', decryptionKeys.join(','));
    }
    if (previousFile !== undefined) {
      parameters.push(previousFile);
    }

    const spawnedProcess = childProcess.spawn(processPath, parameters.map((value) => value.toString(), { cwd: '.' }));

    spawnedProcess.stdout.pipe(outputStream);
    spawnedProcess.stdout.on('end', resolve);

    spawnedProcess.stderr.setEncoding('utf8');
    spawnedProcess.stderr.on('data', (error) => {
      reject(`Error in process:\n> ${processPath} ${parameters.join(' ')}\n${error}`);
    });
  });
}
